# Copyright 2020 Intel Corporation.
#
# This software and the related documents are Intel copyrighted materials,
# and your use of them is governed by the express license under which they
# were provided to you ("License"). Unless the License provides otherwise,
# you may not use, modify, copy, publish, distribute, disclose or transmit
# this software or the related documents without Intel's prior written
# permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

import sys, os, time, sqlite3, requests, subprocess

python_ver = sys.version.split()[0][0:3]
if python_ver == '3.8': 
    import predict_38 as predict 
    import profile_38 as profile
if python_ver == '3.6': 
    import predict_36 as predict 
    import profile_36 as profile

def detect_platform():
    platforms = predict.list_platform()
    cpu_sku, _, _, os_ver, ram_size, _, _, _, _, _= predict.detect_platform()
    valid, platformID, id = False, None, 1
    for platform in platforms:
        if platform[2][0:len(platform[2])-8] == cpu_sku[0:len(cpu_sku)-8]:
            if platform[3][0:7] ==  os_ver[0:7]: 
                if float (platform[4][0:len(platform[4])-3]) > 7.6: 
                    valid , platformID = True, id 
        id+=1
    return {"valid":valid, "detected_platform_id":platformID, "cpu_sku":cpu_sku, "os_ver":os_ver, "ram_size":ram_size}

def reset_params(RI_path, container_name):
    if os.path.isdir(RI_path):
        if find_process (RI_path, container_name): 
            output = subprocess.run(["docker-compose", "-f", RI_path + "/docker-compose.yml","down"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return True
    else: 
        return False

def systemcall_RI(RI_path, container_name):
    output = subprocess.run(["docker-compose", "-f", RI_path + "/docker-compose.yml","up","-d"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    start=find_process(RI_path, container_name)
    trial = 0
    while not start:
        time.sleep(1)    
        start=find_process(RI_path, container_name)
        trial += 1
        if trial == 150: 
            break
    if start:
        trial = 0
        while trial < 60:
            time.sleep(1)
            trial += 1
            fps,ch =  profile_fps()
            if fps != 0: return True
            else: pass  
    return False
	
def find_process(RI_path, container_name):
    found = False
    output = subprocess.run(["docker-compose", "-f", RI_path + "/docker-compose.yml","ps"], stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout.decode("utf-8")
    for line in output.split("\n"):
        if container_name in line.split(): found = True
    return found

def find_pid(container_name):
    pid=[]
    try:
        a = subprocess.check_output(['docker', 'top', container_name]).decode('utf-8').split('\n')
        for line in a[1:]:
            try: 
                pid.append( int(float(line.split()[1] )  ))
            except: pass
    except: pass
    return pid

def profile_muliple_process(container_name):
    pid = find_pid(container_name)
    data = 0
    for p in pid:
        try:
            data+=profile.profiler('CPU_UTIL', p, 1).output[0]
        except: pass
    return data

def profile_fps():
    fps = 0
    try:
        r = requests.get ('http://localhost:5000/fps/all')
        fps_raw = r.json()['fps']
        for ch in fps_raw:
            for k,v in ch.items():
                fps += v
        fps = fps /len(fps_raw)
    except: 
        fps_raw = {}
    channel = len(fps_raw)
    return fps, channel

def predict_fps(average_fps, current_platform_id, target_platform_id, dbdir = 'wlmdb/'):
    files = sorted(os.listdir(dbdir))
    id = 1
    for file in files:
        if file[len(file)-3:len(file)] == '.db':
            if id == current_platform_id: current_config_file = dbdir + file
            if id == target_platform_id: target_config_file = dbdir + file
            id+=1
    db_current = sqlite3.connect(current_config_file)
    current_coef = db_current.execute ("SELECT * FROM fps").fetchall()[0][0]
    db_target = sqlite3.connect(target_config_file)
    target_coef = db_target.execute ("SELECT * FROM fps").fetchall()[0][0]
	
    return target_coef / current_coef * average_fps
	
