# Copyright 2020 Intel Corporation.
#
# This software and the related documents are Intel copyrighted materials,
# and your use of them is governed by the express license under which they
# were provided to you ("License"). Unless the License provides otherwise,
# you may not use, modify, copy, publish, distribute, disclose or transmit
# this software or the related documents without Intel's prior written
# permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

from wlm_module import systemcall_RI, \
    detect_platform, \
    predict_fps, \
    reset_params, \
    profile_muliple_process, \
    profile_fps, \
    find_process

import os, time, statistics, datetime, logging, sys
import flask
from flask import request, jsonify

python_ver = sys.version.split()[0][0:3]
if python_ver == '3.8':
    import predict_38 as predict
    import profile_38 as profile
if python_ver == '3.6':
    import predict_36 as predict
    import profile_36 as profile

app = flask.Flask(__name__)
sys.tracebacklimit = 0

headers = {'X-Content-Type-Options':'nosniff', 'X-XSS-Protection':'1; mode=block', 'Strict-Transport-Security':'max-age=31536000; includeSubDomains'}
container_name = 'itm_smartcity'

logging.basicConfig(filename='wlm.log', filemode='w', format='%(levelname)s - %(message)s')
logging.getLogger().setLevel(logging.DEBUG)
logging.getLogger().addHandler(logging.StreamHandler())

@app.route('/detect_platform', methods = ['GET'])
def detect():
    compatibility = detect_platform()
    return jsonify(compatibility), headers

@app.route('/initialize', methods = ['POST'])
def initialize():
    global RI_time_limit, RI_start_time, cpu_profiled,fps, channel, predictions, running
    
    try:
        logging.info("instance:" + str(running))
        if running is True:
            logging.warning ("Repeated initialize request. An instance is already running.")
        else: running = True
    except: running = True

    RI_start_time = None 
    cpu_profiled, predictions = [], {}
    fps, channel = 0, 0
    cpu_profiled, RI_status = [], False
    params = request.get_json ( silent = True ) 
    RI_path = params['ri_path']
	
    try: RI_time_limit = params['time_limit']
    except: RI_time_limit = 300
	
    os.system("hostname -I | cut -d' ' -f1 > ip")
    with open ("ip", "r") as f:
        ip = f.readline().split()[0]
    os.remove("ip")
    os.environ["HOST_IP"] = ip
    logging.info("RI is using host IP:" + os.environ["HOST_IP"])
    RI_found = reset_params(RI_path, container_name) 

    if not RI_found:
        logging.warning('RI not found')
        return jsonify ({'ri_status':False}), 400 , headers 
    try:
        user_defined_fps = params['fps']
        os.environ["INFER_FPS"] = str(user_defined_fps)
    except: os.environ["INFER_FPS"] = ''
    logging.info("LIMIT FPS to " + os.environ["INFER_FPS"])
	
    RI_status = systemcall_RI( RI_path, container_name)
    if RI_status == True: 
        logging.info('RI_started')
        running = True
    else:
        running = False
        logging.warning('RI cannot start normally. Please check influxdb/grafana connection')
    RI_start_time = time.time()

    return jsonify({'ri_status':RI_status }), headers
	
@app.route('/profile', methods = ['POST'])
def profiling():
    global RI_time_limit, RI_start_time, cpu_profiled, fps, channel, predictions, running
    
    try:
        if running == False:
            err_msg = "Invalid API request. Instance is not initialized."
            logging.warning (err_msg)
            return jsonify ({'ri_status':False}), 400 , headers
    except: 
        err_msg = "Invalid API request. Instance is not initialized."
        logging.warning (err_msg)
        return jsonify ({'ri_status':False}), 400 , headers

    params = request.get_json (silent = True)
    container_name = 'itm_smartcity'
    RI_path = params["ri_path"]
    times_up=False

    try:
        if ( time.time() - RI_start_time ) > RI_time_limit:
            times_up = True  
    except: 
        err_msg = "RI start time is not recorded. Probably because Profile API is called before initialise API or after final_result API."
        logging.warning (err_msg)
        raise Exception (err_msg)

    pid_alive = find_process(RI_path, container_name)
    if params['wlm_func'] == 'predict': 
        if pid_alive: 
            data_now = 	profile_muliple_process(container_name)	
            cpu_profiled.append(data_now)
            fps, channel = profile_fps()
            if fps == 0: logging.warning("RI FPS API is not up...")
            user_platform_id = params['detected_platform_id']	
            target_platform_id = params['target_platform_id']	
            platform_names=predict.list_platform()			
            target = []
            current = {'label' : platform_names[user_platform_id - 1 ][2] , 'cpu' : cpu_profiled , 'fps': fps}			
		
            for platforms in target_platform_id	:
                p = platforms			
                if channel > 6:
                    if user_platform_id == 2: user_platform_id = 4
                    if platforms == 2: p = 4
                try: predictions [platforms].append (predict.predict ( 'CPU_UTIL', user_platform_id, p , data_now )[0] )
                except: predictions [platforms] =  [ predict.predict ( 'CPU_UTIL', user_platform_id, p , data_now )[0] ]
                target.append ( {'label':platform_names[platforms-1][2], 'value' : predictions [platforms] } )
            return jsonify ( {'current':current, 'target':target, 'channel': channel, 'times_up': times_up, 'message':'success' } ), headers
        else:
            logging.info("Simulation time is up...")
            return jsonify ({"times_up": times_up, "message":"RI is not active"})

    elif params['wlm_func'] == 'recommend':
        if pid_alive: 
            data_now = 	profile_muliple_process(container_name)	
            cpu_profiled.append(data_now)
            return jsonify ({"current_cpu":data_now, "times_up": times_up, "message":"success"}), headers
        else:
            logging.info("Simulation time is up...")
            return jsonify ({"times_up": times_up, "message":"RI is not active"}), headers    

@app.route('/final_result', methods= ['POST'])
def result():
    global predictions, running

    if running == False:
        err_msg = "Invalid API request. Instance is not initialized."
        logging.warning (err_msg)
        return jsonify ({'ri_status':False}), 400 , headers
    else: running = False

    data={}
    platform_names=predict.list_platform()	
    params = request.get_json (silent = True)
    user_platform_id = params['detected_platform_id']
    RI_path = params['ri_path']	

    if params['wlm_func'] == 'recommend':
        scores = predict.recommend("CPU_UTIL", user_platform_id, cpu_profiled)
        data['current_platform'] = platform_names[user_platform_id-1][2]
        data['scores'] = scores	
        data['timestamp'] = str(datetime.datetime.now()	)

        reset_params(RI_path, container_name)
        logging.info("Bringing down active RI...")
        return jsonify ({"scores":scores}), headers
		
    elif params['wlm_func'] == 'predict':
        target_platform_id = params['target_platform_id']
        prev_best_score = 0
        data['target'] = []		
        for platforms in target_platform_id	:
            p = platforms
            if channel > 6:
                if user_platform_id == 2: user_platform_id = 4
                if platforms == 2 : p = 4
            cpu_predict = predict.predict('CPU_UTIL', user_platform_id, p, statistics.mean(cpu_profiled))[0]
            fps_predict = predict_fps(fps, user_platform_id, platforms)
            score = fps_predict / cpu_predict
            if score > prev_best_score: 
                best_target_platform = platform_names[platforms-1][2]
                prev_best_score = score
            data ['target'].append({'label': platform_names[platforms-1][2] , 'cpu': cpu_predict , 'fps': fps_predict, 'prev_cpu':predictions [platforms]  })
						
        data['current'] = {'label': platform_names[user_platform_id-1][2] , 'cpu':statistics.mean(cpu_profiled) ,'fps':fps, 'prev_cpu':cpu_profiled}
        data['best_target_platform'] = best_target_platform		
        data['timestamp'] = str(datetime.datetime.now()	)

        reset_params(RI_path, container_name)
        logging.info("Bringing down active RI...")	
        return jsonify( data ), headers

if __name__ == '__main__':
     app.run(host='0.0.0.0', port=7000 )
